# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2017-11-21 03:32
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('report_builder', '0004_auto_20170915_2046'),
    ]

    operations = [
        migrations.AddField(
            model_name='filterfield',
            name='filter_delta',
            field=models.BigIntegerField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='filterfield',
            name='filter_type',
            field=models.CharField(blank=True, choices=[('exact', 'Equals'), ('iexact', 'Equals (case-insensitive)'), ('contains', 'Contains'), ('icontains', 'Contains (case-insensitive)'), ('in', 'in (comma seperated 1,2,3)'), ('gt', 'Greater than'), ('gte', 'Greater than equals'), ('lt', 'Less than'), ('lte', 'Less than equals'), ('startswith', 'Starts with'), ('istartswith', 'Starts with (case-insensitive)'), ('endswith', 'Ends with'), ('iendswith', 'Ends with  (case-insensitive)'), ('range', 'range'), ('relative_range', 'relative_range'), ('week_day', 'Week day'), ('isnull', 'Is null'), ('regex', 'Regular Expression'), ('iregex', 'Reg. Exp. (case-insensitive)'), ('max', 'Max (annotation-filter)'), ('min', 'Min (annotation-filter)')], default='icontains', max_length=20),
        ),
    ]
